/*---------------------------------------------------------+
I                                                          I
I  Module            : modres                              I
I                                                          I
I  Creation          : 22 Mars      1991                   I
I  Derniere revision : 4  Novembre  1997                   I
I                                                          I
I  Auteur            : G. Grimonprez                       I
I                                                          I
I                                                          I
I  INTERFACE                                               I
I                                                          I
I void init_controleur(                                    I
I      int numero_port,        COM1 ou COM2 ou COM3 ou COM4I
I      unsigned long int baud, 1 a 115200                  I
I      int nb_bits_donnee,     5 a 8                       I
I      int nb_bits_arret,      1 ou 2                      I
I      int parite,             SANS, PAIRE ou IMPAIRE      I
I      int max_time            Nombre de secondes          I
I                    );                                    I
I                                                          I
I BOOLEAN  recevoir_octet(char *c);                        I
I                                                          I
I char     reception_octet(void);                          I
I                                                          I
I void     emission_octet(char c);                         I
I                                                          I
I void     restaure_controleur(void);                      I
I                                                          I
I void     modif_timeout(int secondes)                     I
I                                                          I
+---------------------------------------------------------*/

/*---------------------------------------------------------+
I                            INCLUDES                      I
+---------------------------------------------------------*/

#include <stdio.h>
#include <pc.h>
#include <time.h>
#include <stdlib.h>
#include <conio.h>
#include <unistd.h>
#include <svasync.h>
#include "modres.h"

/*---------------------------------------------------------+
I                            DEFINES                       I
+---------------------------------------------------------*/

#define TRUE        1
#define FALSE       0

/*---------------------------------------------------------+
I                      TYPES                               I
+---------------------------------------------------------*/

/*---------------------------------------------------------+
I                      PROTOTYPES                          I
+---------------------------------------------------------*/

void err_init(char *message);
void init_controleur(int numero_port,
		     int baud,int nb_bits_donnee,
		     int nb_bits_arret,int parite,int max_time);
BOOLEAN  recevoir_octet(char *c);
BOOLEAN  recevoir_octet_timeout(char *c);
char     reception_octet(void);
BOOLEAN  emettre_octet(char c);
BOOLEAN  emettre_octet_timeout(char c);
void     emission_octet(char c);
void     restaure_controleur(void);

/*---------------------------------------------------------+
I                      DECLARATIONS GLOBALES               I
+---------------------------------------------------------*/

int max_time_out;

/*---------------------------------------------------------+
I                           FONCTIONS                      I
+---------------------------------------------------------*/

/*------ ERREUR_FATALE ------------------------------------*/

void erreur_fatale(char *message)
{

 printf("\nERREUR FATALE : %s\n",message);
 restaure_controleur();
 exit(1);
}

/*----- MODIF_TIMEOUT ------------------------------------*/

void modif_timeout(int secondes)
{

 max_time_out=secondes;
}

/*----- RESTAURE_CONTROLEUR ------------------------------*/

void restaure_controleur(void)
{

 SVAsyncStop();
}

/*----- ERR_INIT ----------------------------------------*/

void err_init(char *message)
{

 printf("\nERREUR INIT_CONTROLEUR: %s\n",message);
 exit(1);
}

/*----- INIT_CONTROLEUR ---------------------------------*/

void init_controleur(int numero_port,
                     int vitesse,int nb_bits_donnee,
                     int nb_bits_arret,int parite,int max_time)
{

 printf("\nINIT CONTROLEUR:\n\n");
 switch(numero_port)
       {
        case COM1:
        case COM2:
        case COM3:
        case COM4:
             if(SVAsyncInit(numero_port))
                err_init("Impossible d'initialiser le port");
             break;
        default:
             err_init("numero de port different de COM1,COM2,COM3 ou COM4");
             break;
        }
 printf("     Bauds: %lu\n",vitesse);
 switch(nb_bits_donnee) {
        case 7:
        case 8:
               printf("     Nombre de bits de donnees: %d\n",nb_bits_donnee);
               break;
        default:
               err_init("nb bits de donnees different de 7 ou 8");
               break;
        }
 switch(nb_bits_arret) {
        case 1:
        case 2:
               printf("     Nombre de bits d'arret: %d\n",nb_bits_arret);
               break;
        default:
               err_init("nb bits d'arret different de 1 et 2");
               break;
        }
switch(parite) {
       case SANS:
               printf("     sans parite\n");
               break;
       case PAIRE:
               printf("     parite paire\n");
               break;
       case IMPAIRE:
               printf("     parite impaire\n");
               break;
       default:
               err_init("parite differente de SANS et PAIRE et IMPAIRE");
               break;
        }
 printf("     Valeur du time_out %d\n\n",max_time);
 max_time_out=max_time;
 SVAsyncFifoInit();
 SVAsyncSet(vitesse,(nb_bits_donnee-5) | ((nb_bits_arret-1)<<2) | parite);
 // SVAsyncHand(DTR | RTS);
}

/*----- RECEVOIR_OCTET -----------------------------------*/

BOOLEAN recevoir_octet(char *car)
{
 int c;

 c=SVAsyncIn();
 if(c==-1) return FALSE;
 *car=(char)c;
 return TRUE;
}

/*----- EMISSION_OCTET -----------------------------------*/

void emission_octet(char c)
{
 time_t top,w;

 top=time(&top);
 do {
    if(time(&w)-top>max_time_out)
       erreur_fatale("Time Out en emission");
    }
 while(SVAsyncOutStat());
 SVAsyncOut(c);
}

/*----- RECEVOIR_OCTET_TIMEOUT ---------------------------*/

BOOLEAN recevoir_octet_timeout(char *c)
{
 time_t top,w;

 top=time(&top);
 do {
    if(time(&w)-top>max_time_out) return FALSE;
    } while(!recevoir_octet(c));
 return TRUE;
}

/*----- RECEPTION_OCTET ----------------------------------*/

char reception_octet(void)
{
 char c;

 if(!recevoir_octet_timeout(&c)) erreur_fatale("Timeout en reception");
 return c;
}

/*----- LIRE_CLAVIER -------------------------------------*/

BOOLEAN lire_clavier(char *car_clavier,BOOLEAN *b_special)
{

 if(kbhit()) {
    *car_clavier=getch();
    if(*car_clavier==0) {
       *car_clavier=getch();
       *b_special=TRUE;
       }
    else *b_special=FALSE;
    return TRUE;
    }
 return FALSE;
}

